# generated by datamodel-codegen:
#   filename:  v2-normal.json
#   timestamp: 2023-06-09T14:34:47+00:00

from __future__ import annotations

from typing import Any, Dict, List

from pydantic import BaseModel


class Metadata(BaseModel):
    apiVersion: str
    endpointPrefix: str
    jsonVersion: str
    protocol: str
    serviceAbbreviation: str
    serviceFullName: str
    serviceId: str
    signatureVersion: str
    targetPrefix: str
    uid: str


class Http(BaseModel):
    method: str
    requestUri: str


class Input(BaseModel):
    shape: str


class Output(BaseModel):
    shape: str


class Error(BaseModel):
    shape: str


class CreateActivity(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str
    idempotent: bool


class CreateStateMachine(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str
    idempotent: bool


class DeleteActivity(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class DeleteStateMachine(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class DescribeActivity(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class DescribeExecution(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class DescribeMapRun(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class DescribeStateMachine(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class DescribeStateMachineForExecution(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class GetActivityTask(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class GetExecutionHistory(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class ListActivities(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class ListExecutions(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class ListMapRuns(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class ListStateMachines(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class ListTagsForResource(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class SendTaskFailure(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class SendTaskHeartbeat(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class SendTaskSuccess(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class StartExecution(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str
    idempotent: bool


class Endpoint(BaseModel):
    hostPrefix: str


class StartSyncExecution(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str
    endpoint: Endpoint


class StopExecution(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class TagResource(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class UntagResource(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class UpdateMapRun(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str


class UpdateStateMachine(BaseModel):
    name: str
    http: Http
    input: Input
    output: Output
    errors: List[Error]
    documentation: str
    idempotent: bool


class Operations(BaseModel):
    CreateActivity: CreateActivity
    CreateStateMachine: CreateStateMachine
    DeleteActivity: DeleteActivity
    DeleteStateMachine: DeleteStateMachine
    DescribeActivity: DescribeActivity
    DescribeExecution: DescribeExecution
    DescribeMapRun: DescribeMapRun
    DescribeStateMachine: DescribeStateMachine
    DescribeStateMachineForExecution: DescribeStateMachineForExecution
    GetActivityTask: GetActivityTask
    GetExecutionHistory: GetExecutionHistory
    ListActivities: ListActivities
    ListExecutions: ListExecutions
    ListMapRuns: ListMapRuns
    ListStateMachines: ListStateMachines
    ListTagsForResource: ListTagsForResource
    SendTaskFailure: SendTaskFailure
    SendTaskHeartbeat: SendTaskHeartbeat
    SendTaskSuccess: SendTaskSuccess
    StartExecution: StartExecution
    StartSyncExecution: StartSyncExecution
    StopExecution: StopExecution
    TagResource: TagResource
    UntagResource: UntagResource
    UpdateMapRun: UpdateMapRun
    UpdateStateMachine: UpdateStateMachine


class Error26(BaseModel):
    shape: str
    documentation: str


class Cause(BaseModel):
    shape: str
    documentation: str


class Members(BaseModel):
    error: Error26
    cause: Cause


class ActivityFailedEventDetails(BaseModel):
    type: str
    members: Members
    documentation: str


class Member(BaseModel):
    shape: str


class ActivityList(BaseModel):
    type: str
    member: Member


class ActivityArn(BaseModel):
    shape: str
    documentation: str


class Name(BaseModel):
    shape: str
    documentation: str


class CreationDate(BaseModel):
    shape: str
    documentation: str


class Members1(BaseModel):
    activityArn: ActivityArn
    name: Name
    creationDate: CreationDate


class ActivityListItem(BaseModel):
    type: str
    required: List[str]
    members: Members1
    documentation: str


class Members2(BaseModel):
    error: Error26
    cause: Cause


class ActivityScheduleFailedEventDetails(BaseModel):
    type: str
    members: Members2
    documentation: str


class Resource(BaseModel):
    shape: str
    documentation: str


class Input26(BaseModel):
    shape: str
    documentation: str


class InputDetails(BaseModel):
    shape: str
    documentation: str


class TimeoutInSeconds(BaseModel):
    shape: str
    documentation: str
    box: bool


class HeartbeatInSeconds(BaseModel):
    shape: str
    documentation: str
    box: bool


class Members3(BaseModel):
    resource: Resource
    input: Input26
    inputDetails: InputDetails
    timeoutInSeconds: TimeoutInSeconds
    heartbeatInSeconds: HeartbeatInSeconds


class ActivityScheduledEventDetails(BaseModel):
    type: str
    required: List[str]
    members: Members3
    documentation: str


class WorkerName(BaseModel):
    shape: str
    documentation: str


class Members4(BaseModel):
    workerName: WorkerName


class ActivityStartedEventDetails(BaseModel):
    type: str
    members: Members4
    documentation: str


class Output26(BaseModel):
    shape: str
    documentation: str


class OutputDetails(BaseModel):
    shape: str
    documentation: str


class Members5(BaseModel):
    output: Output26
    outputDetails: OutputDetails


class ActivitySucceededEventDetails(BaseModel):
    type: str
    members: Members5
    documentation: str


class Members6(BaseModel):
    error: Error26
    cause: Cause


class ActivityTimedOutEventDetails(BaseModel):
    type: str
    members: Members6
    documentation: str


class Arn(BaseModel):
    type: str
    max: int
    min: int


class BilledDuration(BaseModel):
    type: str
    min: int


class BilledMemoryUsed(BaseModel):
    type: str
    min: int


class BilledMemoryUsedInMB(BaseModel):
    shape: str
    documentation: str


class BilledDurationInMilliseconds(BaseModel):
    shape: str
    documentation: str


class Members7(BaseModel):
    billedMemoryUsedInMB: BilledMemoryUsedInMB
    billedDurationInMilliseconds: BilledDurationInMilliseconds


class BillingDetails(BaseModel):
    type: str
    members: Members7
    documentation: str


class Included(BaseModel):
    shape: str
    documentation: str


class Members8(BaseModel):
    included: Included


class CloudWatchEventsExecutionDataDetails(BaseModel):
    type: str
    members: Members8
    documentation: str


class LogGroupArn(BaseModel):
    shape: str
    documentation: str


class Members9(BaseModel):
    logGroupArn: LogGroupArn


class CloudWatchLogsLogGroup(BaseModel):
    type: str
    members: Members9
    documentation: str


class ConnectorParameters(BaseModel):
    type: str
    max: int
    min: int
    sensitive: bool


class Tags(BaseModel):
    shape: str
    documentation: str


class Members10(BaseModel):
    name: Name
    tags: Tags


class CreateActivityInput(BaseModel):
    type: str
    required: List[str]
    members: Members10


class Members11(BaseModel):
    activityArn: ActivityArn
    creationDate: CreationDate


class CreateActivityOutput(BaseModel):
    type: str
    required: List[str]
    members: Members11


class Definition(BaseModel):
    shape: str
    documentation: str


class RoleArn(BaseModel):
    shape: str
    documentation: str


class Type(BaseModel):
    shape: str
    documentation: str


class LoggingConfiguration(BaseModel):
    shape: str
    documentation: str


class TracingConfiguration(BaseModel):
    shape: str
    documentation: str


class Members12(BaseModel):
    name: Name
    definition: Definition
    roleArn: RoleArn
    type: Type
    loggingConfiguration: LoggingConfiguration
    tags: Tags
    tracingConfiguration: TracingConfiguration


class CreateStateMachineInput(BaseModel):
    type: str
    required: List[str]
    members: Members12


class StateMachineArn(BaseModel):
    shape: str
    documentation: str


class Members13(BaseModel):
    stateMachineArn: StateMachineArn
    creationDate: CreationDate


class CreateStateMachineOutput(BaseModel):
    type: str
    required: List[str]
    members: Members13


class Definition1(BaseModel):
    type: str
    max: int
    min: int
    sensitive: bool


class Members14(BaseModel):
    activityArn: ActivityArn


class DeleteActivityInput(BaseModel):
    type: str
    required: List[str]
    members: Members14


class DeleteActivityOutput(BaseModel):
    type: str
    members: Dict[str, Any]


class Members15(BaseModel):
    stateMachineArn: StateMachineArn


class DeleteStateMachineInput(BaseModel):
    type: str
    required: List[str]
    members: Members15


class DeleteStateMachineOutput(BaseModel):
    type: str
    members: Dict[str, Any]


class Members16(BaseModel):
    activityArn: ActivityArn


class DescribeActivityInput(BaseModel):
    type: str
    required: List[str]
    members: Members16


class Members17(BaseModel):
    activityArn: ActivityArn
    name: Name
    creationDate: CreationDate


class DescribeActivityOutput(BaseModel):
    type: str
    required: List[str]
    members: Members17


class ExecutionArn(BaseModel):
    shape: str
    documentation: str


class Members18(BaseModel):
    executionArn: ExecutionArn


class DescribeExecutionInput(BaseModel):
    type: str
    required: List[str]
    members: Members18


class Status(BaseModel):
    shape: str
    documentation: str


class StartDate(BaseModel):
    shape: str
    documentation: str


class StopDate(BaseModel):
    shape: str
    documentation: str


class InputDetails1(BaseModel):
    shape: str


class OutputDetails1(BaseModel):
    shape: str


class TraceHeader(BaseModel):
    shape: str
    documentation: str


class MapRunArn(BaseModel):
    shape: str
    documentation: str


class Members19(BaseModel):
    executionArn: ExecutionArn
    stateMachineArn: StateMachineArn
    name: Name
    status: Status
    startDate: StartDate
    stopDate: StopDate
    input: Input26
    inputDetails: InputDetails1
    output: Output26
    outputDetails: OutputDetails1
    traceHeader: TraceHeader
    mapRunArn: MapRunArn
    error: Error26
    cause: Cause


class DescribeExecutionOutput(BaseModel):
    type: str
    required: List[str]
    members: Members19


class Members20(BaseModel):
    mapRunArn: MapRunArn


class DescribeMapRunInput(BaseModel):
    type: str
    required: List[str]
    members: Members20


class MaxConcurrency(BaseModel):
    shape: str
    documentation: str


class ToleratedFailurePercentage(BaseModel):
    shape: str
    documentation: str


class ToleratedFailureCount(BaseModel):
    shape: str
    documentation: str


class ItemCounts(BaseModel):
    shape: str
    documentation: str


class ExecutionCounts(BaseModel):
    shape: str
    documentation: str


class Members21(BaseModel):
    mapRunArn: MapRunArn
    executionArn: ExecutionArn
    status: Status
    startDate: StartDate
    stopDate: StopDate
    maxConcurrency: MaxConcurrency
    toleratedFailurePercentage: ToleratedFailurePercentage
    toleratedFailureCount: ToleratedFailureCount
    itemCounts: ItemCounts
    executionCounts: ExecutionCounts


class DescribeMapRunOutput(BaseModel):
    type: str
    required: List[str]
    members: Members21


class Members22(BaseModel):
    executionArn: ExecutionArn


class DescribeStateMachineForExecutionInput(BaseModel):
    type: str
    required: List[str]
    members: Members22


class Definition2(BaseModel):
    shape: str
    documentation: str


class UpdateDate(BaseModel):
    shape: str
    documentation: str


class LoggingConfiguration1(BaseModel):
    shape: str


class Label(BaseModel):
    shape: str
    documentation: str


class Members23(BaseModel):
    stateMachineArn: StateMachineArn
    name: Name
    definition: Definition2
    roleArn: RoleArn
    updateDate: UpdateDate
    loggingConfiguration: LoggingConfiguration1
    tracingConfiguration: TracingConfiguration
    mapRunArn: MapRunArn
    label: Label


class DescribeStateMachineForExecutionOutput(BaseModel):
    type: str
    required: List[str]
    members: Members23


class Members24(BaseModel):
    stateMachineArn: StateMachineArn


class DescribeStateMachineInput(BaseModel):
    type: str
    required: List[str]
    members: Members24


class Members25(BaseModel):
    stateMachineArn: StateMachineArn
    name: Name
    status: Status
    definition: Definition2
    roleArn: RoleArn
    type: Type
    creationDate: CreationDate
    loggingConfiguration: LoggingConfiguration1
    tracingConfiguration: TracingConfiguration
    label: Label


class DescribeStateMachineOutput(BaseModel):
    type: str
    required: List[str]
    members: Members25


class Enabled(BaseModel):
    type: str


class EventId(BaseModel):
    type: str


class Members26(BaseModel):
    error: Error26
    cause: Cause


class ExecutionAbortedEventDetails(BaseModel):
    type: str
    members: Members26
    documentation: str


class Members27(BaseModel):
    error: Error26
    cause: Cause


class ExecutionFailedEventDetails(BaseModel):
    type: str
    members: Members27
    documentation: str


class ExecutionList(BaseModel):
    type: str
    member: Member


class ItemCount(BaseModel):
    shape: str
    documentation: str
    box: bool


class Members28(BaseModel):
    executionArn: ExecutionArn
    stateMachineArn: StateMachineArn
    name: Name
    status: Status
    startDate: StartDate
    stopDate: StopDate
    mapRunArn: MapRunArn
    itemCount: ItemCount


class ExecutionListItem(BaseModel):
    type: str
    required: List[str]
    members: Members28
    documentation: str


class InputDetails2(BaseModel):
    shape: str
    documentation: str


class Members29(BaseModel):
    input: Input26
    inputDetails: InputDetails2
    roleArn: RoleArn


class ExecutionStartedEventDetails(BaseModel):
    type: str
    members: Members29
    documentation: str


class ExecutionStatus(BaseModel):
    type: str
    enum: List[str]


class OutputDetails2(BaseModel):
    shape: str
    documentation: str


class Members30(BaseModel):
    output: Output26
    outputDetails: OutputDetails2


class ExecutionSucceededEventDetails(BaseModel):
    type: str
    members: Members30
    documentation: str


class Members31(BaseModel):
    error: Error26
    cause: Cause


class ExecutionTimedOutEventDetails(BaseModel):
    type: str
    members: Members31
    documentation: str


class Members32(BaseModel):
    activityArn: ActivityArn
    workerName: WorkerName


class GetActivityTaskInput(BaseModel):
    type: str
    required: List[str]
    members: Members32


class TaskToken(BaseModel):
    shape: str
    documentation: str


class Members33(BaseModel):
    taskToken: TaskToken
    input: Input26


class GetActivityTaskOutput(BaseModel):
    type: str
    members: Members33


class MaxResults(BaseModel):
    shape: str
    documentation: str


class ReverseOrder(BaseModel):
    shape: str
    documentation: str


class NextToken(BaseModel):
    shape: str
    documentation: str


class IncludeExecutionData(BaseModel):
    shape: str
    documentation: str


class Members34(BaseModel):
    executionArn: ExecutionArn
    maxResults: MaxResults
    reverseOrder: ReverseOrder
    nextToken: NextToken
    includeExecutionData: IncludeExecutionData


class GetExecutionHistoryInput(BaseModel):
    type: str
    required: List[str]
    members: Members34


class Events(BaseModel):
    shape: str
    documentation: str


class Members35(BaseModel):
    events: Events
    nextToken: NextToken


class GetExecutionHistoryOutput(BaseModel):
    type: str
    required: List[str]
    members: Members35


class Timestamp(BaseModel):
    shape: str
    documentation: str


class Id(BaseModel):
    shape: str
    documentation: str


class PreviousEventId(BaseModel):
    shape: str
    documentation: str


class ActivityFailedEventDetails1(BaseModel):
    shape: str


class ActivityScheduleFailedEventDetails1(BaseModel):
    shape: str
    documentation: str


class ActivityScheduledEventDetails1(BaseModel):
    shape: str


class ActivityStartedEventDetails1(BaseModel):
    shape: str


class ActivitySucceededEventDetails1(BaseModel):
    shape: str


class ActivityTimedOutEventDetails1(BaseModel):
    shape: str


class TaskFailedEventDetails(BaseModel):
    shape: str
    documentation: str


class TaskScheduledEventDetails(BaseModel):
    shape: str
    documentation: str


class TaskStartFailedEventDetails(BaseModel):
    shape: str
    documentation: str


class TaskStartedEventDetails(BaseModel):
    shape: str
    documentation: str


class TaskSubmitFailedEventDetails(BaseModel):
    shape: str
    documentation: str


class TaskSubmittedEventDetails(BaseModel):
    shape: str
    documentation: str


class TaskSucceededEventDetails(BaseModel):
    shape: str
    documentation: str


class TaskTimedOutEventDetails(BaseModel):
    shape: str
    documentation: str


class ExecutionFailedEventDetails1(BaseModel):
    shape: str


class ExecutionStartedEventDetails1(BaseModel):
    shape: str


class ExecutionSucceededEventDetails1(BaseModel):
    shape: str


class ExecutionAbortedEventDetails1(BaseModel):
    shape: str


class ExecutionTimedOutEventDetails1(BaseModel):
    shape: str


class MapStateStartedEventDetails(BaseModel):
    shape: str
    documentation: str


class MapIterationStartedEventDetails(BaseModel):
    shape: str
    documentation: str


class MapIterationSucceededEventDetails(BaseModel):
    shape: str
    documentation: str


class MapIterationFailedEventDetails(BaseModel):
    shape: str
    documentation: str


class MapIterationAbortedEventDetails(BaseModel):
    shape: str
    documentation: str


class LambdaFunctionFailedEventDetails(BaseModel):
    shape: str


class LambdaFunctionScheduleFailedEventDetails(BaseModel):
    shape: str


class LambdaFunctionScheduledEventDetails(BaseModel):
    shape: str


class LambdaFunctionStartFailedEventDetails(BaseModel):
    shape: str
    documentation: str


class LambdaFunctionSucceededEventDetails(BaseModel):
    shape: str
    documentation: str


class LambdaFunctionTimedOutEventDetails(BaseModel):
    shape: str


class StateEnteredEventDetails(BaseModel):
    shape: str


class StateExitedEventDetails(BaseModel):
    shape: str


class MapRunStartedEventDetails(BaseModel):
    shape: str
    documentation: str


class MapRunFailedEventDetails(BaseModel):
    shape: str
    documentation: str


class Members36(BaseModel):
    timestamp: Timestamp
    type: Type
    id: Id
    previousEventId: PreviousEventId
    activityFailedEventDetails: ActivityFailedEventDetails1
    activityScheduleFailedEventDetails: ActivityScheduleFailedEventDetails1
    activityScheduledEventDetails: ActivityScheduledEventDetails1
    activityStartedEventDetails: ActivityStartedEventDetails1
    activitySucceededEventDetails: ActivitySucceededEventDetails1
    activityTimedOutEventDetails: ActivityTimedOutEventDetails1
    taskFailedEventDetails: TaskFailedEventDetails
    taskScheduledEventDetails: TaskScheduledEventDetails
    taskStartFailedEventDetails: TaskStartFailedEventDetails
    taskStartedEventDetails: TaskStartedEventDetails
    taskSubmitFailedEventDetails: TaskSubmitFailedEventDetails
    taskSubmittedEventDetails: TaskSubmittedEventDetails
    taskSucceededEventDetails: TaskSucceededEventDetails
    taskTimedOutEventDetails: TaskTimedOutEventDetails
    executionFailedEventDetails: ExecutionFailedEventDetails1
    executionStartedEventDetails: ExecutionStartedEventDetails1
    executionSucceededEventDetails: ExecutionSucceededEventDetails1
    executionAbortedEventDetails: ExecutionAbortedEventDetails1
    executionTimedOutEventDetails: ExecutionTimedOutEventDetails1
    mapStateStartedEventDetails: MapStateStartedEventDetails
    mapIterationStartedEventDetails: MapIterationStartedEventDetails
    mapIterationSucceededEventDetails: MapIterationSucceededEventDetails
    mapIterationFailedEventDetails: MapIterationFailedEventDetails
    mapIterationAbortedEventDetails: MapIterationAbortedEventDetails
    lambdaFunctionFailedEventDetails: LambdaFunctionFailedEventDetails
    lambdaFunctionScheduleFailedEventDetails: LambdaFunctionScheduleFailedEventDetails
    lambdaFunctionScheduledEventDetails: LambdaFunctionScheduledEventDetails
    lambdaFunctionStartFailedEventDetails: LambdaFunctionStartFailedEventDetails
    lambdaFunctionSucceededEventDetails: LambdaFunctionSucceededEventDetails
    lambdaFunctionTimedOutEventDetails: LambdaFunctionTimedOutEventDetails
    stateEnteredEventDetails: StateEnteredEventDetails
    stateExitedEventDetails: StateExitedEventDetails
    mapRunStartedEventDetails: MapRunStartedEventDetails
    mapRunFailedEventDetails: MapRunFailedEventDetails


class HistoryEvent(BaseModel):
    type: str
    required: List[str]
    members: Members36
    documentation: str


class Truncated(BaseModel):
    shape: str
    documentation: str


class Members37(BaseModel):
    truncated: Truncated


class HistoryEventExecutionDataDetails(BaseModel):
    type: str
    members: Members37
    documentation: str


class HistoryEventList(BaseModel):
    type: str
    member: Member
    documentation: str


class HistoryEventType(BaseModel):
    type: str
    enum: List[str]


class Identity(BaseModel):
    type: str
    max: int


class IncludeExecutionData1(BaseModel):
    type: str


class IncludeExecutionDataGetExecutionHistory(BaseModel):
    type: str
    box: bool


class Members38(BaseModel):
    error: Error26
    cause: Cause


class LambdaFunctionFailedEventDetails1(BaseModel):
    type: str
    members: Members38
    documentation: str


class Members39(BaseModel):
    error: Error26
    cause: Cause


class LambdaFunctionScheduleFailedEventDetails1(BaseModel):
    type: str
    members: Members39
    documentation: str


class TaskCredentials(BaseModel):
    shape: str
    documentation: str


class Members40(BaseModel):
    resource: Resource
    input: Input26
    inputDetails: InputDetails2
    timeoutInSeconds: TimeoutInSeconds
    taskCredentials: TaskCredentials


class LambdaFunctionScheduledEventDetails1(BaseModel):
    type: str
    required: List[str]
    members: Members40
    documentation: str


class Members41(BaseModel):
    error: Error26
    cause: Cause


class LambdaFunctionStartFailedEventDetails1(BaseModel):
    type: str
    members: Members41
    documentation: str


class Members42(BaseModel):
    output: Output26
    outputDetails: OutputDetails2


class LambdaFunctionSucceededEventDetails1(BaseModel):
    type: str
    members: Members42
    documentation: str


class Members43(BaseModel):
    error: Error26
    cause: Cause


class LambdaFunctionTimedOutEventDetails1(BaseModel):
    type: str
    members: Members43
    documentation: str


class Members44(BaseModel):
    maxResults: MaxResults
    nextToken: NextToken


class ListActivitiesInput(BaseModel):
    type: str
    members: Members44


class Activities(BaseModel):
    shape: str
    documentation: str


class Members45(BaseModel):
    activities: Activities
    nextToken: NextToken


class ListActivitiesOutput(BaseModel):
    type: str
    required: List[str]
    members: Members45


class StatusFilter(BaseModel):
    shape: str
    documentation: str


class Members46(BaseModel):
    stateMachineArn: StateMachineArn
    statusFilter: StatusFilter
    maxResults: MaxResults
    nextToken: NextToken
    mapRunArn: MapRunArn


class ListExecutionsInput(BaseModel):
    type: str
    members: Members46


class Executions(BaseModel):
    shape: str
    documentation: str


class Members47(BaseModel):
    executions: Executions
    nextToken: NextToken


class ListExecutionsOutput(BaseModel):
    type: str
    required: List[str]
    members: Members47


class ListExecutionsPageToken(BaseModel):
    type: str
    max: int
    min: int


class Members48(BaseModel):
    executionArn: ExecutionArn
    maxResults: MaxResults
    nextToken: NextToken


class ListMapRunsInput(BaseModel):
    type: str
    required: List[str]
    members: Members48


class MapRuns(BaseModel):
    shape: str
    documentation: str


class Members49(BaseModel):
    mapRuns: MapRuns
    nextToken: NextToken


class ListMapRunsOutput(BaseModel):
    type: str
    required: List[str]
    members: Members49


class Members50(BaseModel):
    maxResults: MaxResults
    nextToken: NextToken


class ListStateMachinesInput(BaseModel):
    type: str
    members: Members50


class StateMachines(BaseModel):
    shape: str


class Members51(BaseModel):
    stateMachines: StateMachines
    nextToken: NextToken


class ListStateMachinesOutput(BaseModel):
    type: str
    required: List[str]
    members: Members51


class ResourceArn(BaseModel):
    shape: str
    documentation: str


class Members52(BaseModel):
    resourceArn: ResourceArn


class ListTagsForResourceInput(BaseModel):
    type: str
    required: List[str]
    members: Members52


class Members53(BaseModel):
    tags: Tags


class ListTagsForResourceOutput(BaseModel):
    type: str
    members: Members53


class CloudWatchLogsLogGroup1(BaseModel):
    shape: str
    documentation: str


class Members54(BaseModel):
    cloudWatchLogsLogGroup: CloudWatchLogsLogGroup1


class LogDestination(BaseModel):
    type: str
    members: Members54
    documentation: str


class LogDestinationList(BaseModel):
    type: str
    member: Member


class LogLevel(BaseModel):
    type: str
    enum: List[str]


class Level(BaseModel):
    shape: str
    documentation: str


class IncludeExecutionData2(BaseModel):
    shape: str
    documentation: str


class Destinations(BaseModel):
    shape: str
    documentation: str


class Members55(BaseModel):
    level: Level
    includeExecutionData: IncludeExecutionData2
    destinations: Destinations


class LoggingConfiguration3(BaseModel):
    type: str
    members: Members55
    documentation: str


class LongArn(BaseModel):
    type: str
    max: int
    min: int


class Index(BaseModel):
    shape: str
    documentation: str


class Members56(BaseModel):
    name: Name
    index: Index


class MapIterationEventDetails(BaseModel):
    type: str
    members: Members56
    documentation: str


class Pending(BaseModel):
    shape: str
    documentation: str


class Running(BaseModel):
    shape: str
    documentation: str


class Succeeded(BaseModel):
    shape: str
    documentation: str


class Failed(BaseModel):
    shape: str
    documentation: str


class TimedOut(BaseModel):
    shape: str
    documentation: str


class Aborted(BaseModel):
    shape: str
    documentation: str


class Total(BaseModel):
    shape: str
    documentation: str


class ResultsWritten(BaseModel):
    shape: str
    documentation: str


class Members57(BaseModel):
    pending: Pending
    running: Running
    succeeded: Succeeded
    failed: Failed
    timedOut: TimedOut
    aborted: Aborted
    total: Total
    resultsWritten: ResultsWritten


class MapRunExecutionCounts(BaseModel):
    type: str
    required: List[str]
    members: Members57
    documentation: str


class Members58(BaseModel):
    error: Error26
    cause: Cause


class MapRunFailedEventDetails1(BaseModel):
    type: str
    members: Members58
    documentation: str


class Members59(BaseModel):
    pending: Pending
    running: Running
    succeeded: Succeeded
    failed: Failed
    timedOut: TimedOut
    aborted: Aborted
    total: Total
    resultsWritten: ResultsWritten


class MapRunItemCounts(BaseModel):
    type: str
    required: List[str]
    members: Members59
    documentation: str


class MapRunLabel(BaseModel):
    type: str


class MapRunList(BaseModel):
    type: str
    member: Member


class Members60(BaseModel):
    executionArn: ExecutionArn
    mapRunArn: MapRunArn
    stateMachineArn: StateMachineArn
    startDate: StartDate
    stopDate: StopDate


class MapRunListItem(BaseModel):
    type: str
    required: List[str]
    members: Members60
    documentation: str


class Members61(BaseModel):
    mapRunArn: MapRunArn


class MapRunStartedEventDetails1(BaseModel):
    type: str
    members: Members61
    documentation: str


class MapRunStatus(BaseModel):
    type: str
    enum: List[str]


class Length(BaseModel):
    shape: str
    documentation: str


class Members62(BaseModel):
    length: Length


class MapStateStartedEventDetails1(BaseModel):
    type: str
    members: Members62
    documentation: str


class MaxConcurrency1(BaseModel):
    type: str
    min: int


class Name9(BaseModel):
    type: str
    max: int
    min: int


class PageSize(BaseModel):
    type: str
    max: int
    min: int


class PageToken(BaseModel):
    type: str
    max: int
    min: int


class ReverseOrder1(BaseModel):
    type: str


class Members63(BaseModel):
    taskToken: TaskToken
    error: Error26
    cause: Cause


class SendTaskFailureInput(BaseModel):
    type: str
    required: List[str]
    members: Members63


class SendTaskFailureOutput(BaseModel):
    type: str
    members: Dict[str, Any]


class Members64(BaseModel):
    taskToken: TaskToken


class SendTaskHeartbeatInput(BaseModel):
    type: str
    required: List[str]
    members: Members64


class SendTaskHeartbeatOutput(BaseModel):
    type: str
    members: Dict[str, Any]


class Members65(BaseModel):
    taskToken: TaskToken
    output: Output26


class SendTaskSuccessInput(BaseModel):
    type: str
    required: List[str]
    members: Members65


class SendTaskSuccessOutput(BaseModel):
    type: str
    members: Dict[str, Any]


class SensitiveCause(BaseModel):
    type: str
    max: int
    min: int
    sensitive: bool


class SensitiveData(BaseModel):
    type: str
    max: int
    sensitive: bool


class SensitiveDataJobInput(BaseModel):
    type: str
    max: int
    sensitive: bool


class SensitiveError(BaseModel):
    type: str
    max: int
    min: int
    sensitive: bool


class Name10(BaseModel):
    shape: str
    documentation: str


class Members66(BaseModel):
    stateMachineArn: StateMachineArn
    name: Name10
    input: Input26
    traceHeader: TraceHeader


class StartExecutionInput(BaseModel):
    type: str
    required: List[str]
    members: Members66


class Members67(BaseModel):
    executionArn: ExecutionArn
    startDate: StartDate


class StartExecutionOutput(BaseModel):
    type: str
    required: List[str]
    members: Members67


class Members68(BaseModel):
    stateMachineArn: StateMachineArn
    name: Name10
    input: Input26
    traceHeader: TraceHeader


class StartSyncExecutionInput(BaseModel):
    type: str
    required: List[str]
    members: Members68


class InputDetails4(BaseModel):
    shape: str


class OutputDetails4(BaseModel):
    shape: str


class BillingDetails1(BaseModel):
    shape: str
    documentation: str


class Members69(BaseModel):
    executionArn: ExecutionArn
    stateMachineArn: StateMachineArn
    name: Name10
    startDate: StartDate
    stopDate: StopDate
    status: Status
    error: Error26
    cause: Cause
    input: Input26
    inputDetails: InputDetails4
    output: Output26
    outputDetails: OutputDetails4
    traceHeader: TraceHeader
    billingDetails: BillingDetails1


class StartSyncExecutionOutput(BaseModel):
    type: str
    required: List[str]
    members: Members69


class InputDetails5(BaseModel):
    shape: str
    documentation: str


class Members70(BaseModel):
    name: Name10
    input: Input26
    inputDetails: InputDetails5


class StateEnteredEventDetails1(BaseModel):
    type: str
    required: List[str]
    members: Members70
    documentation: str


class OutputDetails5(BaseModel):
    shape: str
    documentation: str


class Members71(BaseModel):
    name: Name10
    output: Output26
    outputDetails: OutputDetails5


class StateExitedEventDetails1(BaseModel):
    type: str
    required: List[str]
    members: Members71
    documentation: str


class StateMachineList(BaseModel):
    type: str
    member: Member


class Members72(BaseModel):
    stateMachineArn: StateMachineArn
    name: Name10
    type: Type
    creationDate: CreationDate


class StateMachineListItem(BaseModel):
    type: str
    required: List[str]
    members: Members72
    documentation: str


class StateMachineStatus(BaseModel):
    type: str
    enum: List[str]


class StateMachineType(BaseModel):
    type: str
    enum: List[str]


class Members73(BaseModel):
    executionArn: ExecutionArn
    error: Error26
    cause: Cause


class StopExecutionInput(BaseModel):
    type: str
    required: List[str]
    members: Members73


class Members74(BaseModel):
    stopDate: StopDate


class StopExecutionOutput(BaseModel):
    type: str
    required: List[str]
    members: Members74


class SyncExecutionStatus(BaseModel):
    type: str
    enum: List[str]


class Key(BaseModel):
    shape: str
    documentation: str


class Value(BaseModel):
    shape: str
    documentation: str


class Members75(BaseModel):
    key: Key
    value: Value


class Tag(BaseModel):
    type: str
    members: Members75
    documentation: str


class TagKey(BaseModel):
    type: str
    max: int
    min: int


class TagKeyList(BaseModel):
    type: str
    member: Member


class TagList(BaseModel):
    type: str
    member: Member


class Members76(BaseModel):
    resourceArn: ResourceArn
    tags: Tags


class TagResourceInput(BaseModel):
    type: str
    required: List[str]
    members: Members76


class TagResourceOutput(BaseModel):
    type: str
    members: Dict[str, Any]


class TagValue(BaseModel):
    type: str
    max: int
    min: int


class Members77(BaseModel):
    roleArn: RoleArn


class TaskCredentials1(BaseModel):
    type: str
    members: Members77
    documentation: str


class ResourceType(BaseModel):
    shape: str
    documentation: str


class Members78(BaseModel):
    resourceType: ResourceType
    resource: Resource
    error: Error26
    cause: Cause


class TaskFailedEventDetails1(BaseModel):
    type: str
    required: List[str]
    members: Members78
    documentation: str


class Region(BaseModel):
    shape: str
    documentation: str


class Parameters(BaseModel):
    shape: str
    documentation: str


class TaskCredentials2(BaseModel):
    shape: str
    documentation: str


class Members79(BaseModel):
    resourceType: ResourceType
    resource: Resource
    region: Region
    parameters: Parameters
    timeoutInSeconds: TimeoutInSeconds
    heartbeatInSeconds: HeartbeatInSeconds
    taskCredentials: TaskCredentials2


class TaskScheduledEventDetails1(BaseModel):
    type: str
    required: List[str]
    members: Members79
    documentation: str


class Members80(BaseModel):
    resourceType: ResourceType
    resource: Resource
    error: Error26
    cause: Cause


class TaskStartFailedEventDetails1(BaseModel):
    type: str
    required: List[str]
    members: Members80
    documentation: str


class Members81(BaseModel):
    resourceType: ResourceType
    resource: Resource


class TaskStartedEventDetails1(BaseModel):
    type: str
    required: List[str]
    members: Members81
    documentation: str


class Members82(BaseModel):
    resourceType: ResourceType
    resource: Resource
    error: Error26
    cause: Cause


class TaskSubmitFailedEventDetails1(BaseModel):
    type: str
    required: List[str]
    members: Members82
    documentation: str


class Members83(BaseModel):
    resourceType: ResourceType
    resource: Resource
    output: Output26
    outputDetails: OutputDetails5


class TaskSubmittedEventDetails1(BaseModel):
    type: str
    required: List[str]
    members: Members83
    documentation: str


class Members84(BaseModel):
    resourceType: ResourceType
    resource: Resource
    output: Output26
    outputDetails: OutputDetails5


class TaskSucceededEventDetails1(BaseModel):
    type: str
    required: List[str]
    members: Members84
    documentation: str


class Members85(BaseModel):
    resourceType: ResourceType
    resource: Resource
    error: Error26
    cause: Cause


class TaskTimedOutEventDetails1(BaseModel):
    type: str
    required: List[str]
    members: Members85
    documentation: str


class TaskToken4(BaseModel):
    type: str
    max: int
    min: int


class TimeoutInSeconds3(BaseModel):
    type: str


class Timestamp1(BaseModel):
    type: str


class ToleratedFailureCount1(BaseModel):
    type: str
    min: int


class ToleratedFailurePercentage1(BaseModel):
    type: str
    max: int
    min: int


class TraceHeader4(BaseModel):
    type: str
    max: int
    min: int
    pattern: str


class Enabled1(BaseModel):
    shape: str
    documentation: str


class Members86(BaseModel):
    enabled: Enabled1


class TracingConfiguration3(BaseModel):
    type: str
    members: Members86
    documentation: str


class UnsignedInteger(BaseModel):
    type: str
    min: int


class UnsignedLong(BaseModel):
    type: str
    min: int


class TagKeys(BaseModel):
    shape: str
    documentation: str


class Members87(BaseModel):
    resourceArn: ResourceArn
    tagKeys: TagKeys


class UntagResourceInput(BaseModel):
    type: str
    required: List[str]
    members: Members87


class UntagResourceOutput(BaseModel):
    type: str
    members: Dict[str, Any]


class MaxConcurrency2(BaseModel):
    shape: str
    documentation: str
    box: bool


class ToleratedFailurePercentage2(BaseModel):
    shape: str
    documentation: str
    box: bool


class ToleratedFailureCount2(BaseModel):
    shape: str
    documentation: str
    box: bool


class Members88(BaseModel):
    mapRunArn: MapRunArn
    maxConcurrency: MaxConcurrency2
    toleratedFailurePercentage: ToleratedFailurePercentage2
    toleratedFailureCount: ToleratedFailureCount2


class UpdateMapRunInput(BaseModel):
    type: str
    required: List[str]
    members: Members88


class UpdateMapRunOutput(BaseModel):
    type: str
    members: Dict[str, Any]


class LoggingConfiguration4(BaseModel):
    shape: str
    documentation: str


class TracingConfiguration4(BaseModel):
    shape: str
    documentation: str


class Members89(BaseModel):
    stateMachineArn: StateMachineArn
    definition: Definition2
    roleArn: RoleArn
    loggingConfiguration: LoggingConfiguration4
    tracingConfiguration: TracingConfiguration4


class UpdateStateMachineInput(BaseModel):
    type: str
    required: List[str]
    members: Members89


class Members90(BaseModel):
    updateDate: UpdateDate


class UpdateStateMachineOutput(BaseModel):
    type: str
    required: List[str]
    members: Members90


class IncludedDetails(BaseModel):
    type: str


class Truncated1(BaseModel):
    type: str


class Shapes(BaseModel):
    ActivityFailedEventDetails: ActivityFailedEventDetails
    ActivityList: ActivityList
    ActivityListItem: ActivityListItem
    ActivityScheduleFailedEventDetails: ActivityScheduleFailedEventDetails
    ActivityScheduledEventDetails: ActivityScheduledEventDetails
    ActivityStartedEventDetails: ActivityStartedEventDetails
    ActivitySucceededEventDetails: ActivitySucceededEventDetails
    ActivityTimedOutEventDetails: ActivityTimedOutEventDetails
    Arn: Arn
    BilledDuration: BilledDuration
    BilledMemoryUsed: BilledMemoryUsed
    BillingDetails: BillingDetails
    CloudWatchEventsExecutionDataDetails: CloudWatchEventsExecutionDataDetails
    CloudWatchLogsLogGroup: CloudWatchLogsLogGroup
    ConnectorParameters: ConnectorParameters
    CreateActivityInput: CreateActivityInput
    CreateActivityOutput: CreateActivityOutput
    CreateStateMachineInput: CreateStateMachineInput
    CreateStateMachineOutput: CreateStateMachineOutput
    Definition: Definition1
    DeleteActivityInput: DeleteActivityInput
    DeleteActivityOutput: DeleteActivityOutput
    DeleteStateMachineInput: DeleteStateMachineInput
    DeleteStateMachineOutput: DeleteStateMachineOutput
    DescribeActivityInput: DescribeActivityInput
    DescribeActivityOutput: DescribeActivityOutput
    DescribeExecutionInput: DescribeExecutionInput
    DescribeExecutionOutput: DescribeExecutionOutput
    DescribeMapRunInput: DescribeMapRunInput
    DescribeMapRunOutput: DescribeMapRunOutput
    DescribeStateMachineForExecutionInput: DescribeStateMachineForExecutionInput
    DescribeStateMachineForExecutionOutput: DescribeStateMachineForExecutionOutput
    DescribeStateMachineInput: DescribeStateMachineInput
    DescribeStateMachineOutput: DescribeStateMachineOutput
    Enabled: Enabled
    EventId: EventId
    ExecutionAbortedEventDetails: ExecutionAbortedEventDetails
    ExecutionFailedEventDetails: ExecutionFailedEventDetails
    ExecutionList: ExecutionList
    ExecutionListItem: ExecutionListItem
    ExecutionStartedEventDetails: ExecutionStartedEventDetails
    ExecutionStatus: ExecutionStatus
    ExecutionSucceededEventDetails: ExecutionSucceededEventDetails
    ExecutionTimedOutEventDetails: ExecutionTimedOutEventDetails
    GetActivityTaskInput: GetActivityTaskInput
    GetActivityTaskOutput: GetActivityTaskOutput
    GetExecutionHistoryInput: GetExecutionHistoryInput
    GetExecutionHistoryOutput: GetExecutionHistoryOutput
    HistoryEvent: HistoryEvent
    HistoryEventExecutionDataDetails: HistoryEventExecutionDataDetails
    HistoryEventList: HistoryEventList
    HistoryEventType: HistoryEventType
    Identity: Identity
    IncludeExecutionData: IncludeExecutionData1
    IncludeExecutionDataGetExecutionHistory: IncludeExecutionDataGetExecutionHistory
    LambdaFunctionFailedEventDetails: LambdaFunctionFailedEventDetails1
    LambdaFunctionScheduleFailedEventDetails: LambdaFunctionScheduleFailedEventDetails1
    LambdaFunctionScheduledEventDetails: LambdaFunctionScheduledEventDetails1
    LambdaFunctionStartFailedEventDetails: LambdaFunctionStartFailedEventDetails1
    LambdaFunctionSucceededEventDetails: LambdaFunctionSucceededEventDetails1
    LambdaFunctionTimedOutEventDetails: LambdaFunctionTimedOutEventDetails1
    ListActivitiesInput: ListActivitiesInput
    ListActivitiesOutput: ListActivitiesOutput
    ListExecutionsInput: ListExecutionsInput
    ListExecutionsOutput: ListExecutionsOutput
    ListExecutionsPageToken: ListExecutionsPageToken
    ListMapRunsInput: ListMapRunsInput
    ListMapRunsOutput: ListMapRunsOutput
    ListStateMachinesInput: ListStateMachinesInput
    ListStateMachinesOutput: ListStateMachinesOutput
    ListTagsForResourceInput: ListTagsForResourceInput
    ListTagsForResourceOutput: ListTagsForResourceOutput
    LogDestination: LogDestination
    LogDestinationList: LogDestinationList
    LogLevel: LogLevel
    LoggingConfiguration: LoggingConfiguration3
    LongArn: LongArn
    MapIterationEventDetails: MapIterationEventDetails
    MapRunExecutionCounts: MapRunExecutionCounts
    MapRunFailedEventDetails: MapRunFailedEventDetails1
    MapRunItemCounts: MapRunItemCounts
    MapRunLabel: MapRunLabel
    MapRunList: MapRunList
    MapRunListItem: MapRunListItem
    MapRunStartedEventDetails: MapRunStartedEventDetails1
    MapRunStatus: MapRunStatus
    MapStateStartedEventDetails: MapStateStartedEventDetails1
    MaxConcurrency: MaxConcurrency1
    Name: Name9
    PageSize: PageSize
    PageToken: PageToken
    ReverseOrder: ReverseOrder1
    SendTaskFailureInput: SendTaskFailureInput
    SendTaskFailureOutput: SendTaskFailureOutput
    SendTaskHeartbeatInput: SendTaskHeartbeatInput
    SendTaskHeartbeatOutput: SendTaskHeartbeatOutput
    SendTaskSuccessInput: SendTaskSuccessInput
    SendTaskSuccessOutput: SendTaskSuccessOutput
    SensitiveCause: SensitiveCause
    SensitiveData: SensitiveData
    SensitiveDataJobInput: SensitiveDataJobInput
    SensitiveError: SensitiveError
    StartExecutionInput: StartExecutionInput
    StartExecutionOutput: StartExecutionOutput
    StartSyncExecutionInput: StartSyncExecutionInput
    StartSyncExecutionOutput: StartSyncExecutionOutput
    StateEnteredEventDetails: StateEnteredEventDetails1
    StateExitedEventDetails: StateExitedEventDetails1
    StateMachineList: StateMachineList
    StateMachineListItem: StateMachineListItem
    StateMachineStatus: StateMachineStatus
    StateMachineType: StateMachineType
    StopExecutionInput: StopExecutionInput
    StopExecutionOutput: StopExecutionOutput
    SyncExecutionStatus: SyncExecutionStatus
    Tag: Tag
    TagKey: TagKey
    TagKeyList: TagKeyList
    TagList: TagList
    TagResourceInput: TagResourceInput
    TagResourceOutput: TagResourceOutput
    TagValue: TagValue
    TaskCredentials: TaskCredentials1
    TaskFailedEventDetails: TaskFailedEventDetails1
    TaskScheduledEventDetails: TaskScheduledEventDetails1
    TaskStartFailedEventDetails: TaskStartFailedEventDetails1
    TaskStartedEventDetails: TaskStartedEventDetails1
    TaskSubmitFailedEventDetails: TaskSubmitFailedEventDetails1
    TaskSubmittedEventDetails: TaskSubmittedEventDetails1
    TaskSucceededEventDetails: TaskSucceededEventDetails1
    TaskTimedOutEventDetails: TaskTimedOutEventDetails1
    TaskToken: TaskToken4
    TimeoutInSeconds: TimeoutInSeconds3
    Timestamp: Timestamp1
    ToleratedFailureCount: ToleratedFailureCount1
    ToleratedFailurePercentage: ToleratedFailurePercentage1
    TraceHeader: TraceHeader4
    TracingConfiguration: TracingConfiguration3
    UnsignedInteger: UnsignedInteger
    UnsignedLong: UnsignedLong
    UntagResourceInput: UntagResourceInput
    UntagResourceOutput: UntagResourceOutput
    UpdateMapRunInput: UpdateMapRunInput
    UpdateMapRunOutput: UpdateMapRunOutput
    UpdateStateMachineInput: UpdateStateMachineInput
    UpdateStateMachineOutput: UpdateStateMachineOutput
    includedDetails: IncludedDetails
    truncated: Truncated1


class Model(BaseModel):
    version: str
    metadata: Metadata
    operations: Operations
    shapes: Shapes
    documentation: str



from __future__ import annotations

from enum import Enum
from typing import Optional

from pydantic import BaseModel

from ..utils.json_utils import ingest_json


class ShapeType(Enum):
    BOOLEAN = "boolean"
    ENUM = "enum"
    FLOAT = "float"
    INTEGER = "integer"
    LIST = "list"
    LONG = "long"
    OPERATION = "operation"
    STRING = "string"
    STRUCTURE = "structure"
    TIMESTAMP = "timestamp"


class ShapeMember(BaseModel):
    type: Optional[ShapeType]
    member: Optional[ShapeMember]
    members: Optional[ShapeMember]


class NamedShapeMember(BaseModel):
    __root__: dict[str, Optional[ShapeMember]]


class Shape(BaseModel):
    type: ShapeType
    required: Optional[list[str]]
    members: Optional[ShapeMember]
    pattern: Optional[str]
    # member: Optional[ShapeMember]
    box: Optional[bool]
    sensitive: Optional[bool]
    min: Optional[int]
    max: Optional[int]
    documentation: Optional[str]
    enum: Optional[list[str]]


class Error(BaseModel):
    shape: str


class HTTPInfo(BaseModel):
    method: str
    requestUri: str


class MemberInfo(BaseModel):
    shape: Optional[str]
    type: Optional[str]


class Input(BaseModel):
    type: str
    required: Optional[list[str]]
    members: Optional[dict[str, Optional[MemberInfo]]]


class Output(BaseModel):
    type: str


class Operation(BaseModel):
    input: Optional[Input]
    output: Optional[Output]
    idempotent: Optional[bool]
    endpoint: Optional[dict[str, str]]


class Operations(BaseModel):
    __root__: dict[str, Operation]


class v2MinJson(BaseModel):
    version: float
    metadata: dict
    operations: Operations
    shapes: dict[str, Shape]


def read_source():
    _min_json = "v2-min.json"
    j_min = ingest_json(_min_json)
    return j_min


def build_model():
    j_min = read_source()
    min_model = v2MinJson.parse_obj(j_min)
    return min_model



from __future__ import annotations

from enum import Enum
from functools import partial
from pprint import pprint as _pprint
from typing import Any, Literal, Optional

from pydantic import BaseModel, Field

from ..utils.json_utils import ingest_json

pprint = partial(_pprint, sort_dicts=False)


class ShapeMember(BaseModel):
    __root__: dict[str, Optional[str]]


class Pagination(BaseModel):
    inputToken: str
    outputToken: str
    items: str
    pageSize: str


class Range(BaseModel):
    min: Optional[int]
    max: Optional[int]


class Endpoint(BaseModel):
    hostPrefix: str


class AwsApiService(BaseModel):
    sdkId: str
    arnNamespace: str
    cloudFormationName: str
    cloudTrailEventSource: str
    endpointPrefix: str


class SigV4(BaseModel):
    name: str


class XmlNamespace(BaseModel):
    uri: str


class ServiceTrait(BaseModel):
    service: AwsApiService = Field(..., alias="aws.api#service")
    sigv4: SigV4 = Field(..., alias="aws.auth#sigv4")
    awsJson: dict = Field(..., alias="aws.protocols#awsJson1_0")
    # documentation: str = Field(..., alias="smithy.api#documentation")
    title: str = Field(..., alias="smithy.api#title")
    xmlNamespace: XmlNamespace = Field(..., alias="smithy.api#xmlNamespace")
    # endpointRuleSet: dict = Field(alias="smithy.rules#endpointRuleSet")


class Trait(BaseModel):
    pattern: Optional[str] = Field(alias="aws.api#pattern")
    range: Optional[Range] = Field(alias="aws.api#range")
    idempotent: Optional[dict] = Field(alias="aws.api#idempotent")
    sensitive: Optional[dict] = Field(alias="aws.api#sensitive")
    default: Optional[bool] = Field(alias="aws.api#default")
    httpError: Optional[int] = Field(alias="aws.api#httpError")
    length: Optional[Range] = Field(alias="aws.api#length")
    paginated: Optional[Pagination] = Field(alias="aws.api#paginated")
    error: Optional[str] = Field(alias="aws.api#error")
    endpoint: Optional[Endpoint] = Field(alias="aws.api#endpoint")


class TargetTrait(BaseModel):
    required: Optional[dict[str, str]] = Field(alias="smithy.api#required")
    enumValue: Optional[str] = Field(alias="smithy.api#enumValue")
    default: Optional[Any] = Field(alias="smithy.api#default")
    # documentation: str = Field(..., alias="smithy.api#documentation")


class TraitWithNamedTarget(BaseModel):
    target: str
    traits: Optional[TargetTrait]  # Optional[dict]  # Optional[dict[str, Trait]]


class SpecialShapeMember(BaseModel):
    __root__: dict[str, Optional[TraitWithNamedTarget]]


class TargetReference(BaseModel):
    target: str


class ServiceShape(BaseModel):
    type: Literal["service"]
    version: str
    operations: list[TargetReference]
    traits: ServiceTrait


class ShapeType(Enum):
    BOOLEAN = "boolean"
    ENUM = "enum"
    FLOAT = "float"
    INTEGER = "integer"
    LIST = "list"
    LONG = "long"
    OPERATION = "operation"
    STRING = "string"
    STRUCTURE = "structure"
    TIMESTAMP = "timestamp"


class Shape(BaseModel):
    type: ShapeType
    members: Optional[SpecialShapeMember]
    member: Optional[ShapeMember]
    errors: Optional[list[TargetReference]]
    input: Optional[TargetReference]
    output: Optional[TargetReference]
    traits: Optional[Trait]


class HTTPInfo(BaseModel):
    method: str
    requestUri: str


class MemberInfo(BaseModel):
    shape: Optional[str]
    type: Optional[str]


class ShapeReference(BaseModel):
    shape: str


class Output(BaseModel):
    type: str


class Operation(BaseModel):
    name: str
    http: HTTPInfo
    input: ShapeReference
    output: ShapeReference
    errors: list[ShapeReference]
    # documentation: str
    idempotent: Optional[bool]


class Operations(BaseModel):
    __root__: dict[str, Operation]


class v3Json(BaseModel):
    smithy: float
    metadata: dict
    shapes: dict[str, ServiceShape | Shape]


def read_source():
    _smithy_json = "v3-smithy.json"
    j_smithy = ingest_json(_smithy_json)
    return j_smithy


def check_rehydrate(model: BaseModel, source: dict) -> bool:
    rehydrated = model.dict(by_alias=True, exclude_unset=True)
    return rehydrated == source


def inspect_source():
    source = read_source()
    shape_it = iter(source["shapes"].items())
    next(shape_it)  # Discard service shape
    rest_of_shapes = [*shape_it]
    for s_name, s in rest_of_shapes:
        if "members" in s:
            members_info = s["members"]
            members_vals = list(members_info.values())
            members_vals_keys = [k for d in members_vals for k, v in d.items()]
            check = tuple([k in members_vals_keys for k in ["traits", "target"]])
            if check[0]:
                for subdict in members_vals:
                    if traits_dict := subdict.get("traits", {}):
                        traits_dict.pop("smithy.api#documentation", None)
                        if traits_dict:
                            try:
                                tt = TargetTrait.parse_obj(traits_dict)
                                rehydrate = check_rehydrate(tt, traits_dict)
                                if not rehydrate:
                                    print(f"Did not rehydrate {tt} into {traits_dict}")
                            except Exception as exc:
                                print(f"Raised {exc}")
                            pass
    return


def view_shape_source(shape_name="com.amazonaws.sfn#AWSStepFunctions"):
    j_smithy = read_source()
    return j_smithy["shapes"][shape_name]


def view_shape_info(shape_key: str = "operations", subkey: str | None = None):
    j_smithy = read_source()
    for v in [*j_smithy["shapes"].values()]:
        if v["type"] == "service" and shape_key in v:
            if subkey is None:
                print(v[shape_key])
            else:
                if subkey in v[shape_key]:
                    print(v[shape_key][subkey])


def build_model():
    j_smithy = read_source()
    smithy_model = v3Json.parse_obj(j_smithy)
    return smithy_model


def extract_shapes():
    smithy_model = build_model()
    _shape_iterator = (v for v in smithy_model.shapes.values())
    service_shape = next(_shape_iterator)
    nonservice_shapes = [s for s in _shape_iterator]
    return service_shape, nonservice_shapes


# service_shape, nonservice_shapes = extract_shapes()



"""
Tests for complete model integrity.
"""
from botodto.aws.data_model_v2_min import build_model as build_model_min
from botodto.aws.data_model_v2_min import v2MinJson
from botodto.aws.data_model_v2_norm import build_model as build_model_norm
from botodto.aws.data_model_v2_norm import v2NormalJson
from botodto.aws.data_model_v3 import build_model as build_model_v3
from botodto.aws.data_model_v3 import v3Json


def test_v2_min_value_check():
    model = build_model_min()
    assert isinstance(model, v2MinJson)
    assert model.shapes  # ["ActivityList"].member is not None


def test_v2_norm_ActivityList_value_check():
    model = build_model_norm()
    assert isinstance(model, v2NormalJson)
    shape_member = model.shapes["ActivityList"].member
    assert shape_member.shape == "ActivityListItem"


def test_v3_value_check():
    model = build_model_v3()
    assert isinstance(model, v3Json)
    service_shape = model.shapes["com.amazonaws.sfn#AWSStepFunctions"]
    assert service_shape.type == "service"
    assert len(service_shape.operations) == 26
    assert service_shape.operations[0].target == "com.amazonaws.sfn#CreateActivity"



"""
Tests for complete model integrity.
"""
from botodto.aws.data_model_v2_min import build_model as build_model_min
from botodto.aws.data_model_v2_min import v2MinJson
from botodto.aws.data_model_v2_norm import build_model as build_model_norm
from botodto.aws.data_model_v2_norm import v2NormalJson
from botodto.aws.data_model_v3 import build_model as build_model_v3
from botodto.aws.data_model_v3 import v3Json


def test_v2_min_build():
    model = build_model_min()
    assert isinstance(model, v2MinJson)


def test_v2_norm_build():
    model = build_model_norm()
    assert isinstance(model, v2NormalJson)


def test_v3_build():
    model = build_model_v3()
    assert isinstance(model, v3Json)



r"""
:mod:`botodto` is a Data Transfer Object (DTO) codegen library to produce Pydantic
data models to wrap the AWS SDK library boto3."""

__all__ = ["api", "aws"]

__author__ = "Louis Maddox"
__license__ = "MIT"
__description__ = "Pydantic interface codegen from Amazon Smithy JSON schemas"
__url__ = "https://github.com/lmmx/botodto"
__uri__ = __url__
__email__ = "louismmx@gmail.com"

from . import api, aws



from __future__ import annotations

from typing import Optional

from pydantic import BaseModel

from ..utils.json_utils import ingest_json


class ShapeReference(BaseModel):
    shape: str
    documentation: Optional[str]
    box: Optional[bool]


class Shape(BaseModel):
    type: Optional[str]
    required: Optional[list[str]]
    members: Optional[dict[str, ShapeReference]]
    pattern: Optional[str]
    member: Optional[ShapeReference]
    box: Optional[bool]
    sensitive: Optional[bool]
    min: Optional[int]
    max: Optional[int]
    documentation: Optional[str]
    enum: Optional[list[str]]


class HTTPInfo(BaseModel):
    method: str
    requestUri: str


class MemberInfo(BaseModel):
    shape: Optional[str]
    type: Optional[str]


class Output(BaseModel):
    type: str


class Operation(BaseModel):
    name: str
    http: HTTPInfo
    input: ShapeReference
    output: ShapeReference
    errors: list[ShapeReference]
    documentation: str
    idempotent: Optional[bool]


class Operations(BaseModel):
    __root__: dict[str, Operation]


class v2NormalJson(BaseModel):
    version: float
    metadata: dict
    operations: Operations
    shapes: dict[str, Shape]


def read_source():
    _norm_json = "v2-normal.json"
    j_norm = ingest_json(_norm_json)
    return j_norm


def build_model():
    j_norm = read_source()
    norm_model = v2NormalJson.parse_obj(j_norm)
    return norm_model



import re
from pathlib import Path

from setuptools import find_packages, setup

########################################################################################

NAME = "botodto"
PROJECT_URLS = {
    "Bug Tracker": f"https://github.com/lmmx/{NAME}/issues",
    "Source Code": f"https://github.com/lmmx/{NAME}",
}
CLASSIFIERS = [
    "Development Status :: 4 - Beta",
    "Intended Audience :: Developers",
    "Natural Language :: English",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Programming Language :: Python :: Implementation :: CPython",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Topic :: Internet :: WWW/HTTP",
]
INSTALL_REQUIRES = Path("requirements.txt").read_text().splitlines()
EXTRAS_REQUIRE = {
    "tests": ["coverage[toml]>=5.5", "pytest"],
}
EXTRAS_REQUIRE["dev"] = EXTRAS_REQUIRE["tests"] + ["pre-commit"]
PYTHON_REQUIRES = ">=3.9"
LONG_DESCRIPTION = Path("README.md").read_text()
PACKAGE_DATA = {NAME: ["py.typed"]}

########################################################################################


def local_scheme(version):
    return ""


def version_scheme(version):
    return version.tag.base_version


########################################################################################


META_PATH = Path(__file__).parent.absolute() / "src" / NAME / "__init__.py"
META_FILE = META_PATH.read_text()


def find_meta(meta):
    "Extract __*meta*__ from META_FILE."
    meta_match = re.search(rf"^__{meta}__ = ['\"]([^'\"]*)['\"]", META_FILE, re.M)
    if meta_match:
        return meta_match.group(1)
    raise RuntimeError("Unable to find __{meta}__ string.".format(meta=meta))


if __name__ == "__main__":
    setup(
        name=NAME,
        description=find_meta("description"),
        license=find_meta("license"),
        url=find_meta("url"),
        project_urls=PROJECT_URLS,
        author=find_meta("author"),
        author_email=find_meta("email"),
        maintainer=find_meta("author"),
        maintainer_email=find_meta("email"),
        long_description=LONG_DESCRIPTION,
        long_description_content_type="text/markdown",
        packages=find_packages("src"),
        package_dir={"": "src"},
        package_data=PACKAGE_DATA,
        include_package_data=True,
        zip_safe=False,
        classifiers=CLASSIFIERS,
        use_scm_version={
            "write_to": "version.py",
            "version_scheme": version_scheme,
            "local_scheme": local_scheme,
        },
        setup_requires=["setuptools_scm"],
        entry_points={
            "console_scripts": [
                # "botodto = botodto.cli:main",
            ],
        },
        install_requires=INSTALL_REQUIRES,
        extras_require=EXTRAS_REQUIRE,
        python_requires=PYTHON_REQUIRES,
    )



from pprint import pprint

from ..utils.json_utils import ingest_json

_min_json = "v2-min.json"
_norm_json = "v2-normal.json"
_smithy_json = "v3-smithy.json"

j_min = ingest_json(_min_json)
j_norm = ingest_json(_norm_json)
j_smithy = ingest_json(_smithy_json)


def strip_char(string, char):
    index = string.find(char)
    if index != -1:
        return string[index + 1 :]
    else:
        return string


def list_keys(json, trim_hash=False, pp=False):
    if trim_hash:
        json = [strip_char(j, "#") for j in json]
    if pp:
        pprint(list(json))
    else:
        return list(json)


def get_input(json):
    pprint(json["input"])


def get_output(json):
    pprint(json["output"])


def run_comparison():
    norm_keys = list_keys(j_norm)
    min_keys = list_keys(j_min)
    smithy_keys = list_keys(j_smithy, trim_hash=True)
    return norm_keys, min_keys, smithy_keys
